# Overview
This is a utility for testing/demoing the Wander system. It receives MQTT messages from a broker, displays them in a GUI, and logs all messages to a file. It can also be used as a REST client to receive information from, and change certain settings on the wander controller.

# Running
This is run with Python 3. Before running all dependencies can be installed with `pip install -r requirements.txt`. The program is launched by running main.py.

It is possible to connect on startup. The following is an example: 
- python main.py --address  192.168.90.64 --username "iseniorinc" --password "guest123!" --base-url "http://1.2.3.4/" --filter "iseniorinc/door/011A-0000-0000-0038/#" 
- The username, password, base-url, and filter parameters are optional.

# Parameters
- address: This is the address of the MQTT server
- username: This is the username to authenticate with the MQTT broker.
- password: This is the password to authenticate with the MQTT broker.
- base-url: This is the URL of the wander controller for REST command purposes.
- filter: This is the MQTT topic filter to apply

# Architecture

There are 5 different files that make up this application.  Below are brief descriptions of what each file does:

### main.py

This file kicks off the application.  It starts the MQTT Client, the Logger and the GUI.

### uiview.py

This file configures the view for the application, defining how components should look.  It is used by the uicontroller.py.

### uicontroller.py

This file contains logic for the GUI.  It handles incoming MQTT messages from the mqtt_client.py and determines how to keep track of them and format them for display in the uiview.py.

### mqtt_client.py

The mqtt_client handles the connection to the broker. A function inside the mqtt_client can be called to add a 'handler'. A handler is a function that is set to execute on each received MQTT message. Any number of handlers can be added.

### logger.py

The logger accepts incoming MQTT messages and logs them to a tsv (Tap Separated) file.

# Assorted Notes
- The filter settings in the GUI do not impact logging. All messages received are logged, any message sent on a topic matching the pattern `door/+/event/#` is logged.
- Messages are not logged until the first full message is received, if a small wander message arrives first, it will not be logged
- On some linux systems the program may not work until running `apt-get install python3-tk`
