#!/usr/bin/env python
try:
    import argparse
    import sys
    from mqtt_client import MqttClient
    from ui_controller import UIController
    from logger import Logger
    from rest_client import RestClient
except SyntaxError as ex:
    # sys.stderr.write("MUST BE RUN WITH PYTHON 3.0 or GREATER\n")
    print(ex)
    exit(1)

def main():
    # Handle args
    parser = argparse.ArgumentParser(description='Launches Field Demo Test')
    parser.add_argument('--address', help='The IP address and port of the broker <ip>:<port>')
    parser.add_argument('--username', help='Username for the broker')
    parser.add_argument('--password', help='Password for the broker')
    parser.add_argument('--base-url', help='Set the base url for API calls')
    parser.add_argument('--filter', help='The topic to filter MQTT messages')
    args = parser.parse_args()

    Logger.start()
    if args.base_url:
        RestClient.set_base_url(args.base_url)
    controller = UIController(filter=args.filter)
    if args.address:
        controller.connect_to_broker_address(args=args)
    controller.start_application()

if __name__ == '__main__':
    try:
        main()
        Logger.stop()
    except KeyboardInterrupt:
        Logger.stop()
        sys.exit(0)
