#!/usr/bin/env python
from enum import Enum
import json
import requests

# Enum of valid request types
class RequestTypes(Enum):
    GET = "GET",
    POST = "POST",
    PUT = "PUT",
    DELETE = "DELETE"

class RestClient:
    base_url = "http://0.0.0.0:8080/"

    # Set the url for all calls
    @staticmethod
    def set_base_url(url):
        # Ensure the base ends in a /
        if url[-1] != "/":
            url = url + "/"
        RestClient.base_url = url

    @staticmethod
    def get_base_url():
        return RestClient.base_url 

    @staticmethod
    def build_url(endpoint):
        return RestClient.base_url + endpoint

    # Executes a post request
    @staticmethod
    def post(endpoint, body):
        url = RestClient.build_url(endpoint)
        headers = {'Content-Type': 'application/json'}
        # Ensure body is never None
        if not body or len(body) == 1:
            body = {}
        # Ensure content is a dictionary
        if body:
            if type(body) == type(""):
                body = json.loads(body)         
        return requests.post(url, json=body, headers=headers)

    @staticmethod
    def get(endpoint):
        url = RestClient.build_url(endpoint)
        return requests.get(url)

    @staticmethod
    def put(endpoint, body):
        url = RestClient.build_url(endpoint)
        headers = {'Content-Type': 'application/json'}
        # Ensure body is never None
        if not body or len(body) == 1:
            body = {}
        # Ensure content is a dictionary
        if body:
            if type(body) == type(""):
                body = json.loads(body)         
        return requests.put(url, json=body, headers=headers)

    @staticmethod
    def delete(endpoint):
        url = RestClient.build_url(endpoint)
        return requests.delete(url)

    # Take in request from front end and execute correctly
    @staticmethod
    def make_request(request_type, endpoint, body=None):
        print("BODY:", str(body), flush=True)
        if request_type == "GET":
            return RestClient.get(endpoint)
        elif request_type == "POST":
            return RestClient.post(endpoint, body)
        elif request_type == "PUT":
            return RestClient.put(endpoint, body)
        elif request_type == "DELETE":
            return RestClient.delete(endpoint)

if __name__ == "__main__":
    print(RestClient.get_base_url())
    RestClient.set_base_url("TESTURL.com")
    print(RestClient.get_base_url())
