from tkinter import Tk, ttk, Radiobutton, Label, Listbox, Scrollbar, Frame, Text, Entry, Button, W, simpledialog, END, DISABLED, SINGLE, BOTH, INSERT

class UIView(Tk):
    """
    A class for configuring and displaying the GUI
    of the application.  
    """

    def __init__(self):
        Tk.__init__(self)
        self.geometry("1000x500")
        self.TkDefaultFont = ("Courier", "10")
        self.title("Inovonics Wander Test Toolkit (Professional Edition)")
        self._configure_tabs()
        self._configure_rest_client()
        self._configure_mqtt_client()


    def _configure_tabs(self):
        """Configure the tabs to switch between clients"""
        self.notebook = ttk.Notebook(self)
        self.notebook.pack()
        self.mqtt_client_frame = Frame(self.notebook)
        self.notebook.add(self.mqtt_client_frame, text="Inovonics MQTT Client", sticky="nsew")
        self.rest_client_frame = Frame(self.notebook)
        self.notebook.add(self.rest_client_frame, text="Inovonics REST Client")
        self.notebook.enable_traversal()
        self.notebook.pack(fill = BOTH, expand = True)


    def _configure_rest_client(self):
        """Configure the GUI for the REST client"""
        self._configure_rest_client_request()
        self._configure_rest_client_response()


    def _configure_mqtt_client(self):
        """Configure the GUI for the MQTT client"""
        self._configure_mqtt_client_menu_bar()
        self._configure_mqtt_client_footer()
        self._configure_mqtt_client_message_list()
        self._configure_mqtt_client_message_status()
    

    def _configure_rest_client_request(self):
        """Configure the request portion of the REST client"""
        # Create a frame for the request portion of the client
        self.request_frame = Frame(self.rest_client_frame, padx="10")
        self.request_frame.pack(side = "top", fill = "x")
        # Create a frame for the top part of the request frame
        self.request_top_frame = Frame(self.request_frame)
        self.request_top_frame.pack(side = "top", fill = "x")
        # Add a frame to add the request types to
        self.request_type_frame = Frame(self.request_top_frame)
        self.request_type_frame.pack(side = "left")
        # Add a label for the request types
        self.request_type_label = Label(self.request_type_frame, text = "Request Type:")
        self.request_type_label.pack(side="left")
        # Add a frame for the request URL
        self.request_url_frame = Frame(self.request_top_frame, padx="50")
        self.request_url_frame.pack(side = "left", fill = "x")
        # Add a label for the request URL
        self.request_url_label = Label(self.request_url_frame, text = "Request URL:  ")
        self.request_url_label.pack(side="left")
        # Add an entry field for the request base URL
        self.request_base_url_entry = Entry(self.request_url_frame, width = "30", text = "")
        self.request_base_url_entry.pack(side="left")
        # Add an entry field for the rest of the request URL
        self.request_url_entry = Entry(self.request_url_frame, exportselection = False, width = "80")
        self.request_url_entry.pack(side="left", fill = "x")
        # Add a button to send the request
        self.request_send_button = Button(self.request_frame, width = "20", height = "1", text = "Send")
        self.request_send_button.pack(side = "bottom", pady="5", padx="20", anchor="e")
        # Add a frame for the body of the request
        self.request_body_frame = Frame(self.request_frame)
        self.request_body_frame.pack(side = "bottom", fill = "x")
        # Configure the scrollbar for the body of the request
        self.request_body_scrollbar = Scrollbar(self.request_body_frame, orient="vertical")
        self.request_body_scrollbar.pack(side = "right", fill = "y")
        # Configure the request body field
        self.request_body = Text(self.request_body_frame, width = "50", height = "9", yscrollcommand = self.request_body_scrollbar.set)
        self.request_body.pack(side = "top", fill = "x")
        self.request_body_scrollbar.config(command = self.request_body.yview)
        # Configure the request body field label
        self.request_body_label = Label(self.request_frame, text = "Request Body:")
        self.request_body_label.pack(side = "bottom", anchor="w")


    def _configure_rest_client_response(self):
        """Configure the response portion of the REST client"""
        # Create a frame for the response section
        self.response_frame = Frame(self.rest_client_frame, padx="10", pady="10")
        self.response_frame.pack(side = "bottom", fill = BOTH, expand = True)
        # Create a frame for the top portion of the response section
        self.response_top_frame = Frame(self.response_frame)
        self.response_top_frame.pack(side = "top", fill = "x")
        # Add a label for the response status
        self.response_status_label = Label(self.response_top_frame, text = "Response Status:  ")
        self.response_status_label.pack(side="left")
        # Add a label for the actual response status
        self.response_status = Label(self.response_top_frame, text = "")
        self.response_status.pack(side="left")
        # Add a button to copy responses to the request field
        self.response_copy_button = Button(self.response_frame, width = "20", height = "1", text = "Copy Response to Request")
        self.response_copy_button.pack(side = "bottom", pady="5", padx="20", anchor="e")
        # Add a frame to hold the response body
        self.response_body_frame = Frame(self.response_frame)
        self.response_body_frame.pack(side = "bottom", fill = BOTH, expand = True)
        # Configure the scroll bar for the response body
        self.response_body_scrollbar = Scrollbar(self.response_body_frame, orient="vertical")
        self.response_body_scrollbar.pack(side = "right", fill = "y")
        # Add the text field to show the response body
        self.response_body = Text(
            self.response_body_frame,
            width = "50",
            height = "6",
            exportselection = False,
            yscrollcommand = self.response_body_scrollbar.set)
        self.response_body.pack(side = "bottom", fill = BOTH, expand = True, )
        self.response_body_scrollbar.config(command = self.response_body.yview)
        # Add a label for the response body
        self.response_body_label = Label(self.response_frame, text = "Response Body:")
        self.response_body_label.pack(side = "bottom", anchor = "w")


    def add_request_type(self, name, variable):
        """Add request radio buttons to the request type frame"""
        request_type = Radiobutton(
            self.request_type_frame,
            width = "5", 
            padx = "2",
            text = name,
            value = name, 
            variable = variable
            )
        request_type.pack(side="left")


    def _configure_mqtt_client_menu_bar(self):
        """Configure the "menu bar" at the top of the application."""
        # Create a frame for the menu bar
        self.menu_bar_frame = Frame(self.mqtt_client_frame, padx = "10", pady = "10")
        self.menu_bar_frame.pack(side = "top", fill = "x")
        # Configure filter submit button
        self.mqtt_broker_address_submit_button = Button(self.menu_bar_frame, text = "Connect To Broker", padx = "20")
        self.mqtt_broker_address_submit_button.grid(row = 0, column = 1)
        # Configure the filter frame
        self.mqtt_filter_frame = Frame(self.menu_bar_frame)
        self.mqtt_filter_frame.grid(row = 0, column = 2, padx = 60)
        # Configure the filter label
        self.mqtt_filter_label = Label(self.mqtt_filter_frame, text = "MQTT Topic Filter:")
        self.mqtt_filter_label.grid(row = 0, column = 1, padx = 10)
        # Configure filter entry box
        self.mqtt_filter_entry = Entry(self.mqtt_filter_frame, exportselection = False, width=40)
        self.mqtt_filter_entry.grid(row = 0, column = 2)
        # Configure filter submit button
        self.mqtt_filter_submit_button = Button(self.mqtt_filter_frame, text = "Apply", padx = "20")
        self.mqtt_filter_submit_button.grid(row = 0, column = 3, padx = 10)
        # Configure the live status button
        self.mqtt_live_status_button = Button(
                self.menu_bar_frame,
                state = DISABLED,
                text = "Live Status",
                padx = "20"
            )
        self.mqtt_live_status_button.grid(row = 0, column = 3)


    def _configure_mqtt_client_message_list(self):
        """Configure the list of incoming MQTT messages"""
        # Create a frame for the label, listbox, and scrollbar to live in
        self.message_list_frame = Frame(self.mqtt_client_frame, padx = "10", pady = "10")
        self.message_list_frame.pack(side = "left", fill = BOTH)
        # Configure the listbox label
        self.mqtt_message_list_label = Label(self.message_list_frame, text = "MQTT Incoming Messages")
        self.mqtt_message_list_label.pack(side = "top")
        # Configure the scrollbar for the listbox
        self.mqtt_message_list_scrollbar = Scrollbar(self.message_list_frame, orient="vertical")
        self.mqtt_message_list_scrollbar.pack(side = "right", fill = "y")
        # Configure the listbox
        self.mqtt_message_list = Listbox(
                self.message_list_frame, 
                selectmode = SINGLE,
                width = 85,
                yscrollcommand = self.mqtt_message_list_scrollbar.set,
                exportselection = False
            )
        self.mqtt_message_list.pack(side = "left", fill = "y")
        self.mqtt_message_list_scrollbar.config(command = self.mqtt_message_list.yview)


    def _configure_mqtt_client_message_status(self):
        """Configure the full message status displayed for selected/incoming messages"""
        # Create a frame for the label and and status box
        self.status_list_frame = Frame(self.mqtt_client_frame, padx = "10", pady = "10")
        self.status_list_frame.pack(side = "right", fill = BOTH, expand = True)
        # Configure the status box label
        self.status_list_label = Label(self.status_list_frame, text = "Message Status")
        self.status_list_label.pack(side = "top")
        # Configure the scrollbar for the status list
        self.status_list_scrollbar = Scrollbar(self.status_list_frame, orient="vertical")
        self.status_list_scrollbar.pack(side = "right", fill = "y")
        # Configure the status text
        self.status_list_text = Text(
                self.status_list_frame,
                exportselection = False,
                yscrollcommand = self.status_list_scrollbar.set
            )
        self.status_list_text.pack(fill = BOTH, expand = True)
        self.status_list_scrollbar.config(command = self.status_list_text.yview)


    def _configure_mqtt_client_footer(self):
        """Configure the footer of the application"""
        # Configure the footer frame
        self.footer_frame = Frame(self.mqtt_client_frame, padx = "10", pady = "10")
        self.footer_frame.pack(side = "bottom", fill = "x")
        # Configure the application status label
        self.application_status_label = Label(self.footer_frame, text = "", fg = "grey")
        self.application_status_label.pack(side = "right")
        # Configure the connection status label
        self.mqtt_connection_status_label = Label(self.footer_frame)
        self.mqtt_connection_status_label.pack(side = "left")


class MqttConnectDialog(simpledialog.Dialog):

    def __init__(self, parent, title = None, mqtt_broker_address = None):
        self.mqtt_broker_address = mqtt_broker_address
        super().__init__(parent, title = title)


    def body(self, parent):
        Label(parent, text="Broker Address:").grid(row = 0, padx = 5, pady = 5, sticky = W)
        Label(parent, text="Username (optional):").grid(row = 1, padx = 5, pady = 5, sticky = W)
        Label(parent, text="Password (optional):").grid(row = 2, padx = 5, pady = 5, sticky = W)

        self.address = Entry(parent)
        self.address.insert(END, self.mqtt_broker_address)
        self.username = Entry(parent)
        self.password = Entry(parent, show = "*")

        self.address.grid(row = 0, column = 1, padx = 5)
        self.username.grid(row = 1, column = 1, padx = 5)
        self.password.grid(row = 2, column = 1, padx = 5)
        return self.address


    def apply(self):
        address = self.address.get()
        username = self.username.get()
        password = self.password.get()
        self.result = address, username, password
        from ee.tk import _execute_tk
        _execute_tk(address)